import { Component, Attribute, OnInit, Input, ElementRef } from '@angular/core';
import { ControlContainer, FormGroup, FormControl } from '@angular/forms';

@Component({
	selector: 'app-checkbutton-group',
	templateUrl: './checkbutton-group.component.html',
	styleUrls: ['./checkbutton-group.component.scss']
})
export class CheckbuttonGroupComponent implements OnInit {

	@Input()
	labels: Array<string>;

	@Input()
	meta = {};

	controlNames: Array<string>;
	isDisabled: Array<boolean>;
	extraClass: string;

	firstControl: FormControl;

	constructor(
		private el: ElementRef,
		private cc: ControlContainer,
		@Attribute('firstEnablesRest') private firstEnablesRest
	) {
		// Read the classes applied to the host element, to pass through to the template
		this.extraClass = el.nativeElement.className;
		this.firstEnablesRest = firstEnablesRest === ''; // True if 'firstEnablesRest' exists as component attribute
	}

	ngOnInit() {
		this.controlNames = Object.keys(this.cc.control.value);
		const labelShortfall = this.controlNames.length - this.labels.length;
		if (labelShortfall > 0) {
			this.labels = [ ...this.labels, ...Array(labelShortfall).fill('LABEL MISSING') ];
		}
		if (this.firstEnablesRest) {
			this.firstControl = (<FormGroup>this.cc.control).controls[this.controlNames[0]] as FormControl;
			this.isDisabled = [false, ...Array(this.controlNames.length - 1).fill(!this.firstControl.value)];
			// Observe value changes on first control
			this.firstControl.valueChanges.subscribe(val => {
				this.isDisabled = [false, ...Array(this.controlNames.length - 1).fill(!val)];
			});
		} else {
			this.isDisabled = Array(this.controlNames.length).fill(false);
		}
	}

}
