import { ComponentFactoryResolver, ComponentRef, Directive, Input, OnChanges, OnInit, Type, ViewContainerRef } from '@angular/core';
import { FormControl } from '@angular/forms';

import * as formFieldComponents from './../components/fields';

@Directive({
	// tslint:disable-next-line:directive-selector
	selector: '[dynafield]'
})
export class DynafieldDirective implements OnInit, OnChanges {

	@Input()
	control: FormControl;

	@Input()
	meta: StringMap;

	component: ComponentRef<any>; // TODO: Tighten up on 'any'

	constructor(
		private resolver: ComponentFactoryResolver,
		private container: ViewContainerRef
	) {}

	ngOnInit() {
		const type = `${this.meta.type}Component`;
		if (!formFieldComponents[type]) {
			const validComponentTypes = Object.keys(formFieldComponents).join(', ');
			throw new Error(
				`Dynaform Dynafield Error: Invalid field type: ${type}.
			 	 Supported types: ${validComponentTypes}`
			);
		}
		const component = this.resolver.resolveComponentFactory<any>(formFieldComponents[type]);
		this.component = this.container.createComponent(component);
		this.component.instance.control = this.control;
		this.component.instance.meta = this.meta;

		// Now move meta variables up a level, for direct access in templates implementing dynaField
		Object.keys(this.meta).map(p => this[p] = this.meta[p]);
	}

	ngOnChanges() {
		if (this.component) {
			this.component.instance.meta = this.meta;
			this.component.instance.control = this.control;
		}
	}

}

